﻿package
{
	import de.polygonal.ds.*;
	
	import flash.events.*;
	import flash.display.*;
	import flash.text.*;
	import flash.utils.*;
	
	public class GraphBuilder extends MovieClip
	{
		public static var MyriadProFont:Font = new MyriadPro();
		
		public var graph:Graph;
		
		private var nodeCount:int = 0;
		
		private var nodeQue:LinkedQueue;
		
		private var target0:Node;
		private var target1:Node;
		
		private var traverse:Boolean;
		
		private var modeTf:TextField;
		
		private var timer:Timer;
		
		private var nodeCanvas:Sprite;
		
		private var pos:Array = [];
		private var arc:Array = [];
		
		public function GraphBuilder()
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			
			graph = new Graph(20);
			
			nodeQue = new LinkedQueue();
			
			traverse = false;
			timer = new Timer(1000, 1);
			
			addChild(nodeCanvas = new Sprite());
			
			modeTf = new TextField();
			modeTf.defaultTextFormat = new TextFormat(GraphBuilder.MyriadProFont.fontName, 11, 0, false, false, null, null, null, "center");
			modeTf.selectable = false;
			modeTf.background = true;
			modeTf.embedFonts = true;
			modeTf.antiAliasType = "advanced";
			modeTf.text = "link mode, select start node";
			modeTf.autoSize = "left";
			addChild(modeTf);
			
			setupGraph();
		}
		
		private function setupGraph():void
		{
			var nodes:Array = [[295,131],[317,251],[379,174],[240,203],[161,104],[124,206],[185,305],[424,278]];
			var arcs:Array = [[2,0],[2,1],[1,3],[0,3],[3,4],[5,3],[5,6],[6,1],[7,1],[1,2],[5,4],[4,0],[2,7]];
			
			for (var i:int = 0; i < nodes.length; i++)
			{
				var x:int = nodes[i][0];
				var y:int = nodes[i][1];
				
				var node:Node = new Node(i);
				nodeCanvas.addChild(node);
				node.x = x;
				node.y = y;
				node.addEventListener(MouseEvent.MOUSE_DOWN, onNodePress);
				
				graph.addNode(node, i);
			}
			
			nodeCount = nodes.length;
			
			for (i = 0; i < arcs.length; i++)
				graph.addArc(arcs[i][0], arcs[i][1], 1);
			
			draw();
		}
		
		private function onKeyDown(e:KeyboardEvent):void
		{
			switch (e.keyCode)
			{
				//'a' add node
				case 65:
					
					if (traverse) return;
					addNode();
					break;
					
				//'t' traverse nodes
				case 84:
					
					//quit if in link mode
					if (target0) return;
					
					modeTf.text = "traverse mode, select node";
					target0 = target1 = null;
					traverse = true;
					break;
				
				//'r' reset graph
				case 82:
					
					//quit if in link mode or traversing
					if (target0 || traverse) return;
					
					target0 = target1 = null;
					
					graph.clear();
					nodeQue.clear();
					nodeCount = 0;
					
					graphics.clear();
					removeChild(nodeCanvas);
					addChild(nodeCanvas = new Sprite());
					
					break;
			}
		}
		
		private function animate(e:TimerEvent):void
		{
			if (nodeQue.size > 0)
			{
				var gn:GraphNode = nodeQue.dequeue();
				
				var node:Node = gn.data;
				node.mark();
				
				drawNodeArcs(gn);
				modeTf.text = "traversing..." + nodeQue.size;
			}
		}
		
		private function onAnimationComplete(e:TimerEvent):void
		{
			traverse = false;
			modeTf.text = "link mode, select start node";
			
			freezeNodes(false);
			unmarkNodes();
			
			target0 = target1 = null;
			draw();
		}
		
		private function onNodePress(e:MouseEvent):void
		{
			var node:Node;
			if (e.target is TextField)
				node = e.target.parent as Node;
			else
				node = e.target as Node;
			
			/*************************************************************
			 * traverse mode
			 *************************************************************/
			if (traverse)
			{
				//quit if animation is running
				if (timer.running) return;
				
				freezeNodes(true);
				
				//apply the breadthFirst algorithm to the clicked node and store
				//the result in a queue to animate it.
				nodeQue.clear();
				
				//unmark all nodes first
				graph.clearMarks();
				graph.breadthFirst(graph.nodes[node.graphIndex], function (gn:GraphNode) { nodeQue.enqueue(gn);});
				
				graphics.lineStyle(2, 0xf40095, 1);
				
				timer = new Timer(500, nodeQue.size + 1 + 4);
				timer.addEventListener(TimerEvent.TIMER, animate);
				timer.addEventListener(TimerEvent.TIMER_COMPLETE, onAnimationComplete);
				timer.start();
				return;
			}
			
			/*************************************************************
			 * link mode
			 *************************************************************/
			if (!target0)
			{
				//first click; store starting node 
				target0 = node;
				modeTf.text = "select target node";
			}
			else
			{
				//second click; store destination node
				target1 = node;
				
				//add arc from start->destination node
				graph.addArc(target0.graphIndex, target1.graphIndex);
				
				arc.push([target0.graphIndex, target1.graphIndex]);
				
				//unmark nodes
				target0.unmark();
				target1.unmark();
				target0 = target1 = null;
				
				//refresh node arcs
				draw();
				
				modeTf.text = "link mode, select start node";
			}
		}
		
		private function addNode():void
		{
			if (nodeCount > graph.maxSize) return;
			
			//create a node sprite
			var node:Node = new Node(nodeCount);
			nodeCanvas.addChild(node);
			node.x = stage.mouseX;
			node.y = stage.mouseY;
			node.addEventListener(MouseEvent.MOUSE_DOWN, onNodePress, false, 100);
			
			pos.push([node.x, node.y]);
			
			//store it inside the graph at index nodeCount
			graph.addNode(node, nodeCount);
			nodeCount++;
			
			//just to make sure nodes don't overlap
			var t:Number = getTimer();
			var overlaps:Boolean = false;
			do
			{
				overlaps = spread();
				
				//bail out
				if (getTimer() - t > 100) break;
			}
			while (overlaps)
			
			//draw all arcs by calling the breadthFirst algorithm on each node
			draw();
		}
		
		private function draw():void
		{
			graphics.clear();
			graphics.lineStyle(2, 0x999999, 1);
			
			//draw all arcs originating from the given node
			for (var i:int = 0; i < graph.nodes.length; i++)
				if (graph.nodes[i]) graph.breadthFirst(graph.nodes[i], drawNodeArcs);
		}
		
		private function drawNodeArcs(node:GraphNode):void
		{
			var arcStart:Node = node.data;
			
			for (var i:int = 0; i < node.numArcs; i++)
			{
				var arc:GraphArc = node.arcs[i];
				var arcTarget:Node = arc.node.data;
				
				var dx:Number = arcTarget.x - arcStart.x;
				var dy:Number = arcTarget.y - arcStart.y;
				var len:Number = Math.sqrt(dx * dx + dy * dy);
				
				var xdir:Number = dx / len;
				var ydir:Number = dy / len;
				
				drawArrow(graphics, arcStart.x + xdir * (Node.RADIUS + 4), arcStart.y + ydir * (Node.RADIUS + 4),
						  xdir, ydir, len - 2 * (Node.RADIUS + 4), 4);
			}
		}
		
		private function freezeNodes(flag:Boolean):void
		{
			//enable/disable mouse events for all nodes
			for (var i:int = 0; i < nodeCanvas.numChildren; i++)
			{
				var node:Node = nodeCanvas.getChildAt(i) as Node;
				node.freeze(flag);
			}
		}
		
		private function unmarkNodes():void
		{
			//reset all nodes
			for (var i:int = 0; i < nodeCanvas.numChildren; i++)
			{
				var node:Node = nodeCanvas.getChildAt(i) as Node;
				node.unmark();
			}
		}
		
		private function spread():Boolean
		{
			var overlaps:Boolean = false;
			for (var i:int = 0; i < nodeCanvas.numChildren - 1; i++)
			{
				var nodeA:Node = nodeCanvas.getChildAt(i) as Node;
				
				for (var j:int = i + 1; j < nodeCanvas.numChildren; j++)
				{
					var nodeB:Node = nodeCanvas.getChildAt(j) as Node;
					
					var dx:Number = nodeA.x - nodeB.x + .1;
					var dy:Number = nodeA.y - nodeB.y + .1;
					if ((dx * dx + dy * dy) <= (3 * Node.RADIUS) * (3 * Node.RADIUS))
					{
						var dist:Number = Math.sqrt(dx * dx + dy * dy);
						var xdir:Number = dx / dist;
						var ydir:Number = dy / dist;
						nodeA.x += xdir * (3 * Node.RADIUS - dist);
						nodeA.y += ydir * (3 * Node.RADIUS - dist);
						overlaps = true;
					}
				}
			}
			return overlaps;
		}
		
		private function drawArrow(g:Graphics, x:Number, y:Number, xdir:Number, ydir:Number, len:Number, size:Number)
		{
			var t:Number = 1 / Math.sqrt(xdir * xdir + ydir * ydir);
			
			var ex:Number = (xdir * t * len) + x;
			var ey:Number = (ydir * t * len) + y;
			
			g.moveTo(x, y);
			g.lineTo(ex, ey);
			
			var dx:Number = (ex - x);
			var dy:Number = (ey - y);
			
			var l:Number = Math.sqrt((dx * dx) + (dy * dy));
			if (l <= 0)	return;
			
			dx /= l;
			dy /= l;
			var nx:Number = -dy;
			var ny:Number =  dx;
			g.moveTo(ex, ey);
			g.lineTo((ex - (size * dx)) - (size * nx), (ey - (size * dy)) - (size * ny));
			g.moveTo(ex, ey);
			g.lineTo((ex - (size * dx)) + (size * nx), (ey - (size * dy)) + (size * ny));
		}
	}
}
